﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using CameraServer;
using System.IO;
using System.Threading;
using System.Net.Sockets;
using System.Windows.Forms ;

namespace CameraClient
{
    /// <summary>
    /// It subscribes a broadcasting generated from a selected video capture device. 
    /// It shows it on PictureBox control
    /// </summary>
    public class Subscriber
    {
        #region LocalVariables
            VideoService vs = null;
            string ip = string.Empty;
            int port = 399;
            bool noDelay = false;
            int receiveTimeOut = 5000;
            int receiveBufferSize = 20000;
            bool isNetworkStrActive = true;
            TcpClient tcpClient = null;
        #endregion

        #region Properties

            /// <summary>
            /// Gets or Sets Delay when Send or Receive Buffers of TCP Client are not full.
            /// Default value is false
            /// </summary>
            public bool NoDelay 
        {
            get 
            {
                return noDelay; 
            }

            set 
            {
                noDelay = value;
            }
        }

            /// <summary>
            /// Gets or Sets amount of time System.NET.Sockets.TcpClient to receive data once a read option is initiated.
            /// Default value is 5000 mili secs
            /// </summary>
            public int ReceiveTimeOut
        {
            get 
            {
                return receiveTimeOut;
            }

            set 
            {
                receiveTimeOut = value;
            }
        }

            /// <summary>
            /// Gets or Sets amount of buffer size to read from network stream.
            /// Defalut value is 20000 mili secs
            /// </summary>
            public int ReceiveBufferSize 
        {
            get 
            {
                return receiveTimeOut;
            }

            set 
            {
                receiveTimeOut = value;
            }
        }
            
            /// <summary>
            /// Sets an IPAddress where broacasting is taking place
            /// </summary>
            public string IPAddress 
            {
                set 
                {
                    ip = value;
                }
            }

            /// <summary>
            /// Sets a Port where broacasting is taking place. Default value is 399
            /// </summary>
            public int Port 
            {
                set 
                {
                    port = value;
                }
            }

        #endregion EndProperties

        /// <summary>
        /// Generates a stream of bytes to send and receive data at given ip and port
        /// </summary>
        /// <returns>System.Net.Sockets.NetworkStream</returns>
        private NetworkStream GetImageStream() 
        {
            if (ip == string.Empty) 
            {
                return null;
            }
            try
            {
                tcpClient = new TcpClient(ip, port);
                tcpClient.NoDelay = noDelay;
                tcpClient.ReceiveTimeout = receiveTimeOut;
                tcpClient.ReceiveBufferSize = receiveBufferSize;
            }
            catch 
            {
                return null;
            }
           
            return tcpClient.GetStream();
        }

        /// <summary>
        /// Populates web cam stream on Picture box. 
        /// </summary>
        /// <param name="picBox"></param>
        /// <returns>It returns a string which consists of messages/errors generaed while processing byte stream</returns>
        public string SuscribeImage(PictureBox picBox) 
        {
            NetworkStream networkStream = GetImageStream();
            string s;
                
            int iBytesComing, iBytesRead, iOffset;
            byte[] byLength = new byte[10];
            byte[] byImage = new byte[1000];
            MemoryStream m = new MemoryStream(byImage);

            do
            {
                try
                {
                        // Read the fixed length string that
                        // tells the image size
                        iBytesRead = networkStream.Read(byLength, 0, 10);

                        if (iBytesRead != 10)
                        {
                            isNetworkStrActive = false;
                            return "No response from host";
                        }

                        s = Encoding.ASCII.GetString(byLength);

                        iBytesComing = Convert.ToInt32(s);

                        // Make sure our buffer is big enough
                        if (iBytesComing > byImage.Length)
                        {
                            byImage = new byte[iBytesComing];
                            m = new MemoryStream(byImage);
                            tcpClient.ReceiveBufferSize = iBytesComing + 10;
                        }
                        else
                        {
                            m.Position = 0;
                        }

                        // Read the image
                        iOffset = 0;

                        do
                        {
                            iBytesRead = networkStream.Read(byImage, iOffset, iBytesComing - iOffset);
                            if (iBytesRead != 0)
                            {
                                iOffset += iBytesRead;
                            }
                            else
                            {
                                isNetworkStrActive = false ;
                                return "No response from host";
                                
                            }
                        } while ((iOffset != iBytesComing) && (isNetworkStrActive));


                        if (isNetworkStrActive)
                        {
                            // Write back a byte
                            networkStream.Write(byImage, 0, 1);
                            
                            // Put the image on the screen
                            picBox.Image = new System.Drawing.Bitmap(m);
                                                       
                        }
                    }
                    catch (Exception e)
                    {
                       isNetworkStrActive = false ;
                       return e.Message;
                        
                    }

                } while (isNetworkStrActive);

                networkStream.Close();
                tcpClient.Close();
                networkStream = null;
                tcpClient = null;
                return "";
            }
        }
    }

    
