﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SkyLordClientDbInterface;

namespace SkyLordGateClient.BusinessLogic.Database
{
    class DatabaseHelper
    {
        private static DatabaseHelper instance = new DatabaseHelper();
        public ClientDbInterfaces InterfaceDb = new SkyLordClientDbInterface.ClientDbInterfaces(); // database object

        /// <summary>
        /// 
        /// </summary>
        private DatabaseHelper() 
        {
            // creating event handlers
            InterfaceDb.UserChanged += new EventHandler(InterfaceDb_UserChanged);
            InterfaceDb.TabChanged += new EventHandler(InterfaceDb_TabChanged);

            // load temporary data in the database
                
             /*  // creating tables in database            
               InterfaceDb.CreateTableUsers();
               InterfaceDb.CreateTableTabs();
               InterfaceDb.CreateTableUserTab();
               InterfaceDb.CreateTableUserConversation();
               InterfaceDb.CreateTableUserTabConnect();            
            
               // creating 2 tabs
               List<int> tabIDs = new List<int>(2);
               TabInfo tabInfo = new TabInfo(); 
               TabInfo tab = new TabInfo();
               tab.TabName = "My Office";
               InterfaceDb.AddTab(tab, out tabInfo);
               tabIDs.Add(tabInfo.TabID);

               tabInfo = new TabInfo();
               tab = new TabInfo();
               tab.TabName = "Kitchen";
               InterfaceDb.AddTab(tab, out tabInfo);
               tabIDs.Add(tabInfo.TabID);

               // creating 2 users
               UserInfo userInfo = new UserInfo();
               UserInfo user = new UserInfo();
               user.FamilyName = "Asghar";
               user.FirstName = "Arooj";
               user.IP = "255.255.255.255";
               user.Deleted = false;
               user.ShiftToServerHour = 2;
               user.IsAdmin = true;
               user.NickName = "AA";            
               InterfaceDb.AddUser(user, out userInfo);
            
               // add this user to tabs
               InterfaceDb.AddUserToTabs(userInfo.UserID, tabIDs);
                     
               userInfo = new UserInfo();
               user = new UserInfo();
               user.FamilyName = "";
               user.FirstName = "Admin";
               user.IP = "255.255.255.255";
               user.Deleted = false;
               user.ShiftToServerHour = 2;
               user.IsAdmin = true;
               user.NickName = "Admin";
               InterfaceDb.AddUser(user, out userInfo);

               // add this user to tabs
               InterfaceDb.AddUserToTabs(userInfo.UserID, tabIDs);    */
        }

        /// <summary>
        /// 
        /// </summary>
        public static DatabaseHelper Instance 
        {
            get 
            {
                return instance;
            }
        }

        /// <summary>
        /// Returns the user nickname from database for the given uID.
        /// </summary>
        /// <returns></returns>
        public string GetUserName(int uID)
        {
            UserInfo u = new UserInfo();
            InterfaceDb.GetUser(uID, out u);
            return u.NickName;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="uID"></param>
        /// <returns></returns>
        public string GetUserIPAddress(int uID) 
        {
            UserInfo u = new UserInfo();
            InterfaceDb.GetUser(uID, out u);
            return u.IP;
        }
        
        /// <summary>
        /// 
        /// </summary>
        /// <param name="uID"></param>
        /// <returns></returns>
        public int GetUserVideoPort(int uID)
        {
            UserInfo u = new UserInfo();
            InterfaceDb.GetUser(uID, out u);
            return u.VideoCommunicationPort;
        }

        /// <summary>
        /// Returns tab names for user
        /// </summary>
        /// <returns></returns>
        public List<string> GetTabNames(int userID)
        {
            List<TabInfo> tabs = new List<TabInfo>();
            string errorMessage = InterfaceDb.GetTabsForUser(userID, out tabs);

            if (tabs == null)
            {
                throw new Exception(errorMessage);
            }

            List<string> tabNames = new List<string>();

            for (int i = 0; i < tabs.Count; i++)
                tabNames.Add(((TabInfo)tabs[i]).TabName);

            return tabNames;
        }

        /// <summary>
        /// Returns User IDs list for this tab
        /// </summary>
        /// <param name="tabIndexOnGUI"></param>
        /// <returns></returns>
        public List<int> GetUsersForTab(int userID, string tabName)
        {
            List<int> userIDs = new List<int>();

            // search for tabID for this tabName in this user's tabs (assuming a user will have tabs with unique names. no repetitions.)
            int tabID = GetTabID(userID, tabName);

            if (tabID != -1)
            {
                List<UserInfo> usersInfo = new List<UserInfo>();
                InterfaceDb.GetUsersForTab(tabID, out usersInfo);

                foreach (UserInfo u in usersInfo)
                    if (u.UserID != userID)// only add users other than this user to the list
                        userIDs.Add(u.UserID);
            }            

            return userIDs;
        }

        /// <summary>
        /// Called when UserAdded event envoked at database.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void InterfaceDb_UserChanged(object sender, EventArgs e)
        {
            UserChangedEventArgs eUserChanged = (UserChangedEventArgs)e;
            if (eUserChanged.changeType == ChangeType.Added)
                Console.WriteLine("TestInterface receives the UserAdded event.");
        }

        /// <summary>
        /// Called when TabAdded event envoked at database.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void InterfaceDb_TabChanged(object sender, EventArgs e)
        {
            TabChangedEventArgs eTabChanged = (TabChangedEventArgs)e;
            if (eTabChanged.changeType == ChangeType.Added)
                Console.WriteLine("TestInterface receives the TabAdded event.");
        }

        /// <summary>
        /// Returns tab id for user, tabname
        /// </summary>
        /// <param name="userID"></param>
        /// <param name="tabName"></param>
        /// <returns></returns>
        public int GetTabID(int userID, string tabName) 
        {
            // search for tabID for this tabName in this user's tabs (assuming a user will have tabs with unique names. no repetitions.)

            int tabID = -1;           
            List<TabInfo> tabs = new List<TabInfo>();
            InterfaceDb.GetTabsForUser(userID, out tabs);

            for (int i = 0; i < tabs.Count; i++)
                if (tabs[i].TabName == tabName)
                {
                    tabID = tabs[i].TabID;
                    break;
                }

            return tabID;
        }

        /// <summary>
        /// Returns IDs of users that are currently connected to tab
        /// </summary>
        /// <param name="tabID"></param>
        /// <returns></returns>
        public List<int> GetConnectedUsersOnTab(int userID, string tabName) 
        {
            int tabID = GetTabID(userID, tabName);
            List<int> userIDs = new List<int>();
            List<UserInfo> connectedUsers = new List<UserInfo>();
            InterfaceDb.GetUsersTabConnected(tabID, out connectedUsers);
            
            for (int i = 0; i < connectedUsers.Count; i++)
                userIDs.Add(connectedUsers[i].UserID);
            
            return userIDs;
        }
    }
}
