﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using SkyLordClientDbInterface;
using SkyLordGateClient.BusinessLogic.Database;

namespace SkyLordGateClient.BusinessLogic.TextChat
{
    class TextChatHelper : IChatCallback
    {
        ChatClient proxy;

        /// <summary>
        /// 
        /// </summary>
        public TextChatHelper()
        {            
            InstanceContext site = new InstanceContext(this);
            proxy = new ChatClient(site);
        }

        /// <summary>
        /// Makes a connection to the server that specifies that the logged in user has connected 
        /// to tabname provided in parameters.
        /// </summary>
        /// <param name="tabName">Name of the tab to connect to.</param>
        public void ConnectToTab(int userID, string tabName)
        {
            BusinessLogicManager.Instance.WriteLogMessage(" In .ConnectToTab() method");

            UserTabInfo userTab = new UserTabInfo();
            List<TabInfo> tabs = new List<TabInfo>();
            DatabaseHelper.Instance.InterfaceDb.GetTabsForUser(userID, out tabs);

            foreach (TabInfo tab in tabs)
            {
                if (tab.TabName == tabName)
                    userTab = new UserTabInfo(userID, tab.TabID);
            }

            proxy.Join(userTab);
        }

        /// <summary>
        /// Tells the server that the user has disconnected from this tab.
        /// </summary>
        /// <param name="tabName"></param>
        public void LeaveTab(int userID, string tabName)
        {
            BusinessLogicManager.Instance.WriteLogMessage(" In .LeaveTab() method");

            UserTabInfo userTab = new UserTabInfo();
            List<TabInfo> tabs = new List<TabInfo>();
            DatabaseHelper.Instance.InterfaceDb.GetTabsForUser(userID, out tabs);

            foreach (TabInfo tab in tabs)
            {
                if (tab.TabName == tabName)
                    userTab = new UserTabInfo(userID, tab.TabID);
            }

            proxy.Leave(userTab);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="userID"></param>
        /// <param name="oldTabName"></param>
        /// <param name="newTabName"></param>
        public void SwitchTabs(int userID, string oldTabName, string newTabName) 
        {
            BusinessLogicManager.Instance.WriteLogMessage(" In .SwitchTabs() method");

            int newTabID = -1;
            UserTabInfo userTab = new UserTabInfo();
            List<TabInfo> tabs = new List<TabInfo>();
            DatabaseHelper.Instance.InterfaceDb.GetTabsForUser(userID, out tabs);

            foreach (TabInfo tab in tabs)
            {
                if (tab.TabName == oldTabName)
                    userTab = new UserTabInfo(userID, tab.TabID);
                else if (tab.TabName == newTabName)
                    newTabID = tab.TabID;
            }

            proxy.SwitchTabs(userTab, newTabID);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="message"></param>
        public void Receive(UserTabInfo sender, UserConversationInfo message)
        {
            lock (this)
            {
                BusinessLogicManager.Instance.WriteLogMessage(" In .Receive() method");
                BusinessLogicManager.Instance.RecieveTextMessage(sender.UserID, message.TabID, message.text);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="userTab"></param>
        public void UserEnter(SkyLordClientDbInterface.UserTabInfo userTab)
        {
            lock (this)
            {
                BusinessLogicManager.Instance.WriteLogMessage(" In .UserEnter() method");
                BusinessLogicManager.Instance.ChangeUserStatus(userTab.UserID, userTab.TabID, true);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="userTab"></param>
        public void UserLeave(SkyLordClientDbInterface.UserTabInfo userTab)
        {
            lock (this)
            {
                BusinessLogicManager.Instance.WriteLogMessage(" In .UserLeave() method");
                BusinessLogicManager.Instance.ChangeUserStatus(userTab.UserID, userTab.TabID, false);
            }
        }

        /// <summary>
        /// Sends a text message from this user
        /// </summary>
        /// <param name="userID"></param>
        /// <param name="message"></param>
        public void SendTextMessage(int senderID, int tabID, int receiverID, string message) 
        {
            lock (this)
            {
                BusinessLogicManager.Instance.WriteLogMessage(" In .SendTextMessage() method: SenderID = " + senderID + " ReceiverID = " + receiverID + " TabID = " + tabID +" Message = " + message);

                UserConversationInfo msg = new UserConversationInfo(senderID, tabID, receiverID, DateTime.Now, message);
                proxy.Say(msg);
            }
        }
    }
}
