﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Configuration;
using SkyLordClientDbInterface;
using SkyLordGateClient.BusinessLogic;

namespace SkyLordGateClient.UserInterface
{
    /// <summary>
    /// Class MainForm
    /// </summary>
    public partial class MainForm : Form
    {
        /// <summary>
        /// Constructor for Class MainForm
        /// </summary>
        public MainForm()
        {
            try
            {

                // Show the user/password authorization form
                AuthorizationForm a = new AuthorizationForm();

                // If user info is loaded successfully from database go ahead and load the GUI components
                if (a.ShowDialog() == DialogResult.OK)
                {
                    InitializeComponent();

                    // add tabpages                
                    List<string> tabNames = BusinessLogicManager.Instance.GetTabNames();
                    for (int i = 0; i < tabNames.Count; i++)
                        tabsTabControl.TabPages.Add(tabNames[i]);

                    BusinessLogicManager.Instance.SetSelectedTabName(tabNames[0]);

                    // add a panel to each tabpage
                    foreach (TabPage tp in tabsTabControl.TabPages)
                    {
                        Panel panel = new Panel();
                        panel.AutoScroll = true;
                        panel.Location = new System.Drawing.Point(tp.Location.X + 2, tp.Location.X + 2);
                        panel.Size = new System.Drawing.Size(tabsTabControl.Size.Width, tabsTabControl.Size.Height);
                        panel.Dock = System.Windows.Forms.DockStyle.Fill;
                        tp.Controls.Add(panel);
                    }

                    // load users on the first tabpage
                    LoadUsersOnTabPage(0);
                    BusinessLogicManager.Instance.ConnectToTab();

                    this.Text = "SkyLord Gate Client - Welcome " + BusinessLogicManager.Instance.GetUserName(BusinessLogicManager.Instance.GetUserID()) + "!";
                }
                else
                {
                    //temporary
                    MessageBox.Show("User initialization failed. Please try again!");
                    Environment.Exit(1);                    
                }
            }
            catch (Exception e) 
            {
                MessageBox.Show(e.Message);                
                Environment.Exit(1);
            }
        }

        /// <summary>
        /// Opens Add New User form on this menu item click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void usersToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AddNewUserForm a = new AddNewUserForm();
            a.ShowDialog();
        }

        /// <summary>
        /// Opens the Audio Video Settings dialog.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void audioVideoSettingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AudioVideoSettingsForm a = new AudioVideoSettingsForm();
            a.ShowDialog();
        }

        /// <summary>
        /// Opens the shared folder in a new window
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void sharedFolderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string sharedFolder = ConfigurationManager.AppSettings["SharedFolder"];
            System.Diagnostics.Process prc = new System.Diagnostics.Process();
            prc.StartInfo.FileName = sharedFolder;
            prc.Start();
        }

        /// <summary>
        /// Loads Users on the selected tabpage
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tabsTabControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            // load user video controls on tabpage
            LoadUsersOnTabPage(tabsTabControl.SelectedIndex);

            // switch tab connection with server
            BusinessLogicManager.Instance.SwitchTabs(tabsTabControl.TabPages[tabsTabControl.SelectedIndex].Text);
        }

        /// <summary>
        /// Prompts user if she wants to navigate away from the tab.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
       /* private void tabsTabControl_Selecting(object sender, TabControlCancelEventArgs e)
        {
            if(e.TabPage.Name != BusinessLogicManager.Instance.GetSelectedTabName())
                if (DialogResult.No == MessageBox.Show("Do you want to navigate away from " + BusinessLogicManager.Instance.GetSelectedTabName() + "  tab?", "Change tab?", MessageBoxButtons.YesNo))
                    e.Cancel = true;                
        }*/
 
        /// <summary>
        /// Loads user video chat controls on each tab page.
        /// </summary>
        /// <param name="tabID">tab page index</param>
        private void LoadUsersOnTabPage(int tabPageID)
        {
            Hashtable usersOnline = BusinessLogicManager.Instance.GetUsersForTab(tabsTabControl.TabPages[tabPageID].Text);            
            List<VideoChatControl> vList = new List<VideoChatControl>();

            Panel p = (Panel)tabsTabControl.TabPages[tabsTabControl.SelectedIndex].Controls[0];
            p.Controls.Clear(); //delete the old controls

            int i = 0;
            foreach(int userID in usersOnline.Keys)
            {
                VideoChatControl vControl = new VideoChatControl(userID,(bool)usersOnline[userID]);
                // dynamically decide location
                int temp = i - (4 * ((int)(Math.Floor((decimal)i / 4)))); // display 4 videochat controls in one row
                int x = (tabsTabControl.Location.X + 50) + (temp * 380);
                int y = tabsTabControl.Location.Y + ((int)(Math.Floor((decimal)i / 4)) * 300);
                vControl.Location = new System.Drawing.Point(x, y);
                vControl.Size = new System.Drawing.Size(308, 287);

                p.Controls.Add(vControl);
                vList.Add(vControl);
                i++;
            }

            BusinessLogicManager.Instance.SetVideoChatControlList(ref vList);            
        }

    }

}
