﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Configuration;
using SkyLordGateClient.BusinessLogic;
using CameraClient;

namespace SkyLordGateClient.UserInterface
{
    public partial class VideoChatControl : UserControl
    {
        int userID = -1; // identifier for this control.
        
        Color originalColor;    // original color of the panel. before blinking.
        Thread blinkThread;  // thread object for blinking.

        Subscriber s = null;
        Thread videoThread = null;

        string userIPAddress = "";
        int videoPort = 0;

        /// <summary>
        /// Constructor for VideoChatControl custom control
        /// </summary>
        /// <param name="uID">Sets the identifier for this control. It is the same as userID of this user in the database.</param>
        public VideoChatControl(int uID, bool isOnline)
        {
            InitializeComponent();
            userID = uID;
            userNameLabel.Text = BusinessLogicManager.Instance.GetUserName(uID);
            userIPAddress = BusinessLogicManager.Instance.GetUserIPAddress(uID);
            videoPort = BusinessLogicManager.Instance.GetUserVideoPort(uID);
            originalColor = mediaPanel.BackColor;
            SetStatus(isOnline);
            pictureBox1.SizeMode = PictureBoxSizeMode.StretchImage;
        }

        /// <summary>
        /// Sets the status to online or not and enables controls accordingly
        /// </summary>
        /// <param name="isOnline"></param>
        public void SetStatus(bool isOnline) 
        {
            messageTextBox.Enabled = isOnline;
            sendButton.Enabled = isOnline;

            if (isOnline)
            {
                ThreadStart ts = new ThreadStart(RunSubscribe);
                videoThread = new Thread(ts);
                videoThread.Start();
            }
            else 
            {
                if (videoThread != null)
                {
                    videoThread.Abort();
                    System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(VideoChatControl));
                    pictureBox1.Image = ((System.Drawing.Image)(resources.GetObject("pictureBox1.Image")));
                }
            }
        }

        /// <summary>
        /// Returns userID.
        /// </summary>
        /// <returns></returns>
        public int GetUserID()
        {
            return userID;
        }

        /// <summary>
        /// Called when the username change event occurs at the database.
        /// </summary>
        /// <param name="name">new name</param>
        public void SetUserNameLabel(string name)
        {
            userNameLabel.Text = name;
        }

        /// <summary>
        /// Starts UI control blinking thread
        /// </summary>
        private void StartBlinking()
        {
            try
            {
                ThreadStart starter = new ThreadStart(Blink);
                blinkThread = new Thread(starter);
                blinkThread.Start();
            }
            catch (Exception)
            {
            }
        }

        /// <summary>
        /// Stops UI control blinking thread
        /// </summary>
        private void StopBlinking()
        {
            try
            {
                blinkThread.Abort();            
            }
            catch (Exception)
            {
            }

        }

        /// <summary>
        /// Makes media panel blink.
        /// </summary>
        private void Blink()
        {
            try
            {
                while (true)
                {
                    mediaPanel.BackColor = Color.Orange;
                    System.Threading.Thread.Sleep(500);
                    mediaPanel.BackColor = originalColor;
                    System.Threading.Thread.Sleep(500);
                }
            }
            catch (Exception)
            {
            }
        }

        /// <summary>
        /// Event handler for key press on message text box.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void messageTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == 13)
            {
                SendTextMessage();  
            }
        }

        /// <summary>
        /// Event handler for send button click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void sendButton_Click(object sender, EventArgs e)
        {
            SendTextMessage();
        }

        
        /// <summary>
        /// Sends text message to server and appends message in chat text box.
        /// </summary>
        /// <param name="message"></param>
        public void SendTextMessage()
        {
            string message = messageTextBox.Text.Trim();
            if (message != "")
            {
                if (chatTextBox.Text != "")
                    chatTextBox.AppendText(System.Environment.NewLine);
                chatTextBox.AppendText(BusinessLogicManager.Instance.GetUserName(BusinessLogicManager.Instance.GetUserID()) + ": " + message);
                BusinessLogicManager.Instance.SendTextMessage(userID, message);
            }
            messageTextBox.Text = "";
        }

        /// <summary>
        /// Receives messages from the server and displays in chat text box.
        /// </summary>
        /// <param name="message"></param>
        public void ReceiveTextMessage(string message) 
        {
            if (chatTextBox.Text != "")
                chatTextBox.AppendText(System.Environment.NewLine);
            chatTextBox.AppendText(userNameLabel.Text + ": " + message);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pictureBox1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (MessageBox.Show("Do you want to start a video conversation with this user?", "Start video chat?", MessageBoxButtons.OKCancel) == DialogResult.OK)
            {
                // start video chat, open audio conversation.  
            }
        }

        /// <summary>
        /// 
        /// </summary>
        private void RunSubscribe()
        {
            s = new Subscriber();
            s.IPAddress = userIPAddress;
            s.Port = videoPort; // temporary hardcoding until value is retrieved from database
            
            s.SuscribeImage(pictureBox1);
        }
    }
}