﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.Runtime.Serialization;

namespace SkyLordClientDbInterface
{
    [DataContract]
    [Table(Name = "Users")]
    public partial class UserInfo
    {
        [DataMember]
        [Column(IsPrimaryKey = true)]
        public int UserID;
        [DataMember]
        [Column]
        public bool IsAdmin;
        [DataMember]
        [Column]
        public string FamilyName;
        [DataMember]
        [Column]
        public string FirstName;
        [DataMember]
        [Column]
        public string IP;
        [DataMember]
        [Column]
        public string Location;
        [DataMember]
        [Column]
        public int ShiftToServerHour;
        [DataMember]
        [Column]
        public bool Deleted;
        [DataMember]
        [Column]
        public string UserPwd;
        [DataMember]
        [Column]
        public string NickName;
        [DataMember]
        [Column]
        public int VideoCommunicationPort;

        public UserInfo(int UserID, bool IsAdmin, string FamilyName, string FirstName, string IP, 
            string Location, int ShiftToServerHour, bool Deleted, string UserPwd, string NickName, int VideoPort)
        {
            this.UserID = UserID;
            this.IsAdmin = IsAdmin;
            this.FamilyName = FamilyName;
            this.FirstName = FirstName;
            this.IP = IP;
            this.Location = Location;
            this.ShiftToServerHour = ShiftToServerHour;
            this.Deleted = Deleted;
            this.UserPwd = UserPwd;
            this.NickName = NickName;
            this.VideoCommunicationPort = VideoPort;
        }
        public UserInfo()
        {
            this.UserID = -1;
            this.IsAdmin = false;
            this.FamilyName = String.Empty;
            this.FirstName = String.Empty;
            this.IP = String.Empty;
            this.Location = String.Empty;
            this.ShiftToServerHour = 0;
            this.Deleted = false;
            this.UserPwd = String.Empty;
            this.NickName = String.Empty;
            this.VideoCommunicationPort = -1;
        }
        public void Assign(UserInfo ui)
        {
            this.UserID = ui.UserID;
            this.IsAdmin = ui.IsAdmin;
            this.FamilyName = ui.FamilyName;
            this.FirstName = ui.FirstName;
            this.IP = ui.IP;
            this.Location = ui.Location;
            this.ShiftToServerHour = ui.ShiftToServerHour;
            this.Deleted = ui.Deleted;
            this.UserPwd = ui.UserPwd;
            this.NickName = ui.NickName;
            this.VideoCommunicationPort = ui.VideoCommunicationPort;
        }
        public bool IsEqual(UserInfo ui)
        {
            if (this.IsAdmin == ui.IsAdmin &&
                this.FamilyName == ui.FamilyName &&
                this.FirstName == ui.FirstName &&
                this.IP == ui.IP &&
                this.Location == ui.Location &&
                this.ShiftToServerHour == ui.ShiftToServerHour &&
                this.Deleted == ui.Deleted &&
                this.UserPwd == ui.UserPwd &&
                this.NickName == ui.NickName &&
                this.VideoCommunicationPort == ui.VideoCommunicationPort)
                return true;
            else
                return false;
        }
    }

    [DataContract]
    [Table(Name = "Tabs")]
    public class TabInfo
    {
        [DataMember]
        [Column(IsPrimaryKey = true)]
        public int TabID;
        [DataMember]
        [Column]
        public string TabName;
        [DataMember]
        [Column]
        public Boolean Deleted;

        public TabInfo(int TabID, string TabName, bool Deleted)
        {
            this.TabID = TabID;
            this.TabName = TabName;
            this.Deleted = Deleted;
        }
        public TabInfo()
        {
            this.TabID = -1;
            this.TabName = String.Empty;
            this.Deleted = false;
        }
        public void Assign(TabInfo ui)
        {
            this.TabID = ui.TabID;
            this.TabName = ui.TabName;
            this.Deleted = ui.Deleted;
        }
        public bool IsEqual(TabInfo ui)
        {
            if (this.TabName == ui.TabName)
                return true;
            else
                return false;
        }
    }

    [DataContract]
    [Table(Name = "UserTab")]
    public partial class UserTabInfo
    {
        [DataMember]
        [Column(IsPrimaryKey = true)]
        public int UserID;
        [DataMember]
        [Column(IsPrimaryKey = true)]
        public int TabID;

        public UserTabInfo(int UserID, int TabID)
        {
            this.TabID = TabID;
            this.UserID = UserID;
        }
        public UserTabInfo()
        {
            this.TabID = -1;
            this.UserID = -1;
        }
        public void Assign(UserTabInfo uti)
        {
            this.TabID = uti.TabID;
            this.UserID = uti.UserID;
        }
    }

    [DataContract]
    [Table(Name = "UserTabConnect")]
    public class UserTabConnectInfo
    {
        [DataMember]
        [Column(IsPrimaryKey = true, IsDbGenerated = true)]
        public int id;
        [DataMember]
        [Column]
        public int UserID;
        [DataMember]
        [Column]
        public int TabID;
        [DataMember]
        [Column]
        public DateTime started;
        [DataMember]
        [Column]
        public DateTime terminated;

        public UserTabConnectInfo(int UserID, int TabID, DateTime started, DateTime terminated)
        {
            this.id = 0;
            this.TabID = TabID;
            this.UserID = UserID;
            this.started = started;
            this.terminated = terminated;
        }
        public UserTabConnectInfo()
        {
            this.id = 0;
            this.TabID = -1;
            this.UserID = -1;
            this.started = DateTime.Now;
            this.terminated = new DateTime(1800, 1, 1);
        }
        public void Assign(UserTabConnectInfo utci)
        {
            this.id = utci.id;
            this.TabID = utci.TabID;
            this.UserID = utci.UserID;
            this.started = utci.started;
            this.terminated = utci.terminated;
        }
    }

    [DataContract]
    [Table(Name = "UserConversation")]
    public partial class UserConversationInfo
    {
        [DataMember]
        [Column(IsPrimaryKey = true, IsDbGenerated = true)]
        public int id;
        [DataMember]
        [Column]
        public int UserID;
        [DataMember]
        [Column]
        public int TabID;
        [DataMember]
        [Column]
        public int DestinationID;
        [DataMember]
        [Column]
        public DateTime sent;
        [DataMember]
        [Column]
        public String text;

        public UserConversationInfo(int UserID, int TabID, int DestinationID, DateTime sent, String text)
        {
            this.id = 0;
            this.TabID = TabID;
            this.UserID = UserID;
            this.DestinationID = DestinationID;
            this.sent = sent;
            this.text = text;
        }
        public UserConversationInfo()
        {
            this.id = 0;
            this.TabID = -1;
            this.UserID = -1;
            this.DestinationID = -1;
            this.sent = DateTime.Now;
            this.text = String.Empty;
        }
        public void Assign(UserConversationInfo uci)
        {
            this.id = uci.id;
            this.TabID = uci.TabID;
            this.UserID = uci.UserID;
            this.DestinationID = uci.DestinationID;
            this.sent = uci.sent;
            this.text = uci.text;
        }
    }
    
   
    public enum ChangeType
    {
        Added,
        Removed,
        Modified,
    }

    /// <summary>
    /// List all public Interfaces to work with Client DataBase.
    /// </summary>
    interface IClientDb
    {
        string CreateTableUsers();
        string CreateTableTabs();
        string CreateTableUserTab();
        string CreateTableUserTabConnect();
        string CreateTableUserConversation();

        string AddUser(UserInfo user, out UserInfo UserInfo);
        string ChangeUser(UserInfo user, out UserInfo UserInfo);
        string RemoveUser(string FamilyName, string FirstName);
        string RemoveUser(int UserID);
        string GetAllUsers(out List<UserInfo> Users);
        string GetUsersNotDeleted(out List<UserInfo> Users);
        string GetUser(int UserID, out UserInfo User);
        string GetUsersForTab(int TabID, out List<UserInfo> Users);

        string AddTab(TabInfo tab, out TabInfo TabInfo);
        string ChangeTab(TabInfo tab, out TabInfo TabInfo);
        string RemoveTab(int TabID);
        string RemoveTab(string TabName);
        string GetAllTabs(out List<TabInfo> Tabs);
        string GetTabsNotDeleted(out List<TabInfo> Tabs);
        string GetTab(int TabID, out TabInfo Tab);
        string GetTabsForUser(int UserID, out List<TabInfo> Tabs);

        string AddUserToTabs(int UserID, List<int> TabIDs);
        string RemoveUserFromTabs(int UserID, List<int> TabIDs);

        string UserConnectStart(int UserID, int TabID);
        string UserConnectStart(int UserID, int TabID, DateTime now);
        string UserConnectEnd(int UserID, int TabID);
        string UserConnectEnd(int UserID, int TabID, DateTime now);
        string UserConnectEnd(int userTabConnectID);
        string UserConnectEnd(int userTabConnectID, DateTime now);
        string GetUsersTabConnected(int TabID, out List<UserInfo> users);
        string IsUserTabConnected(int UserID, int TabID, out bool result);
        string GetUserTabConnected(int UserID, out UserTabConnectInfo result);

        string UserConversation(int UserID, int TabID, int DestinationID, string Text);
        string UserConversation(int UserID, int TabID, int DestinationID, string Text, DateTime now);

        // An event that clients can use to be notified whenever User is changed
        event EventHandler UserChanged;

        // An event that clients can use to be notified whenever Tab is changed
        event EventHandler TabChanged;

        // An event that clients can use to be notified whenever UserTab is changed
        event EventHandler UserTabChanged;

        // An event that clients can use to be notified whenever UserTabConnect is changed
        //event EventHandler UserTabConnectChanged;

        // An event that clients can use to be notified whenever UserConversation is changed
        //event EventHandler UserConversationChanged;
    }

    public class UserChangedEventArgs : EventArgs
    {
        public UserInfo userInfo;
        public ChangeType changeType;
        public UserChangedEventArgs(ChangeType ChangeType, UserInfo userInfo)
        {
            this.userInfo = userInfo;
            this.changeType = ChangeType;
        }
    }

    public class TabChangedEventArgs : EventArgs
    {
        public TabInfo tabInfo;
        public ChangeType changeType;
        public TabChangedEventArgs(ChangeType ChangeType, TabInfo tabInfo)
        {
            this.tabInfo = tabInfo;
            this.changeType = ChangeType;
        }
    }

    public class UserTabChangedEventArgs : EventArgs
    {
        public UserTabInfo userTabInfo;
        public ChangeType changeType;
        public UserTabChangedEventArgs(ChangeType ChangeType, UserTabInfo userTabInfo)
        {
            this.userTabInfo = userTabInfo;
            this.changeType = ChangeType;
        }
    }

    /*public class UserTabConnectChangedEventArgs : EventArgs
    {
        public UserTabConnectInfo userTabConnectInfo;
        public ChangeType changeType;
        public UserTabConnectChangedEventArgs(ChangeType ChangeType, UserTabConnectInfo userTabConnectInfo)
        {
            this.userTabConnectInfo = userTabConnectInfo;
            this.changeType = ChangeType;
        }
    }*/

    /*public class UserConversationChangedEventArgs : EventArgs
    {
        public UserConversationInfo userConversationInfo;
        public ChangeType changeType;
        public UserConversationChangedEventArgs(ChangeType ChangeType, UserConversationInfo userConversationInfo)
        {
            this.userConversationInfo = userConversationInfo;
            this.changeType = ChangeType;
        }
    }*/
}
